@@ -21,42 +21,6 @@
  * Boston, MA 02110-1301, USA.
  */
 
-/**
- * SECTION:element-glstereomix
- * @title: glstereomix
- *
- * Combine 2 input streams to produce a stereoscopic output
- * stream. Input views are taken from the left pad and right pad
- * respectively, and mixed according to their timelines.
- *
- * If either input stream is stereoscopic, the approproriate view
- * (left or right) is taken from each stream and placed into the output.
- *
- * The multiview representation on the output is chosen according to
- * the downstream caps.
- *
- * ## Examples
- * |[
- * gst-launch-1.0 -v videotestsrc pattern=ball name=left \
- *     videotestsrc name=right glstereomix name=mix \
- *     left. ! vid/x-raw,width=640,height=480! glupload ! mix.  \
- *     right. ! video/x-raw,width=640,height=480! glupload ! mix.  \
- *     mix. ! video/x-raw'(memory:GLMemory)',multiview-mode=side-by-side ! \
- *     queue ! glimagesink output-multiview-mode=side-by-side
- * ]| Mix 2 different videotestsrc patterns into a side-by-side stereo image and display it.
- * |[
- * gst-launch-1.0 -ev v4l2src name=left \
- *     videotestsrc name=right \
- *     glstereomix name=mix \
- *     left. ! video/x-raw,width=640,height=480 ! glupload ! glcolorconvert ! mix.  \
- *     right. ! video/x-raw,width=640,height=480 ! glupload ! mix.  \
- *     mix. ! video/x-raw'(memory:GLMemory)',multiview-mode=top-bottom ! \
- *     glcolorconvert ! gldownload ! queue ! x264enc ! h264parse ! \
- *     mp4mux ! progressreport ! filesink location=output.mp4
- * ]| Mix the input from a camera to the left view, and videotestsrc to the right view,
- *    and encode as a top-bottom frame packed H.264 video.
- *
- */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -66,25 +30,15 @@
 #define GST_CAT_DEFAULT gst_gl_stereo_mix_debug
 GST_DEBUG_CATEGORY (gst_gl_stereo_mix_debug);
 
-G_DEFINE_TYPE (GstGLStereoMixPad, gst_gl_stereo_mix_pad, GST_TYPE_GL_MIXER_PAD);
-
-static void
-gst_gl_stereo_mix_pad_class_init (GstGLStereoMixPadClass * klass)
-{
-}
-
-static void
-gst_gl_stereo_mix_pad_init (GstGLStereoMixPad * pad)
-{
-}
-
 #define gst_gl_stereo_mix_parent_class parent_class
 G_DEFINE_TYPE (GstGLStereoMix, gst_gl_stereo_mix, GST_TYPE_GL_MIXER);
 
 static GstCaps *_update_caps (GstVideoAggregator * vagg, GstCaps * caps);
-static gboolean _negotiated_caps (GstAggregator * aggregator, GstCaps * caps);
+static gboolean _negotiated_caps (GstVideoAggregator * videoaggregator,
+    GstCaps * caps);
 gboolean gst_gl_stereo_mix_make_output (GstGLStereoMix * mix);
-static gboolean gst_gl_stereo_mix_process_frames (GstGLStereoMix * mixer);
+static gboolean gst_gl_stereo_mix_process_frames (GstGLStereoMix * mixer,
+    GPtrArray * in_frames);
 
 #define DEFAULT_DOWNMIX GST_GL_STEREO_DOWNMIX_ANAGLYPH_GREEN_MAGENTA_DUBOIS
 
@@ -104,13 +58,9 @@ enum
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("video/x-raw(" GST_CAPS_FEATURE_MEMORY_GL_MEMORY "), "
-        "format = (string) RGBA, "
-        "width = " GST_VIDEO_SIZE_RANGE ", "
-        "height = " GST_VIDEO_SIZE_RANGE ", "
-        "framerate = " GST_VIDEO_FPS_RANGE ","
-        "texture-target = (string) 2D"
-        "; "
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE_WITH_FEATURES
+        (GST_CAPS_FEATURE_MEMORY_GL_MEMORY,
+            "RGBA") "; "
         GST_VIDEO_CAPS_MAKE_WITH_FEATURES
         (GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META,
             "RGBA")
@@ -120,13 +70,9 @@ static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink_%u",
     GST_PAD_SINK,
     GST_PAD_REQUEST,
-    GST_STATIC_CAPS ("video/x-raw(" GST_CAPS_FEATURE_MEMORY_GL_MEMORY "), "
-        "format = (string) RGBA, "
-        "width = " GST_VIDEO_SIZE_RANGE ", "
-        "height = " GST_VIDEO_SIZE_RANGE ", "
-        "framerate = " GST_VIDEO_FPS_RANGE ","
-        "texture-target = (string) 2D"
-        "; "
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE_WITH_FEATURES
+        (GST_CAPS_FEATURE_MEMORY_GL_MEMORY,
+            "RGBA") "; "
         GST_VIDEO_CAPS_MAKE_WITH_FEATURES
         (GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META,
             "RGBA")
@@ -140,6 +86,11 @@ static gboolean gst_gl_stereo_mix_start (GstAggregator * agg);
 static gboolean gst_gl_stereo_mix_src_query (GstAggregator * agg,
     GstQuery * query);
 
+static void
+gst_gl_stereo_mix_find_best_format (GstVideoAggregator * vagg,
+    GstCaps * downstream_caps, GstVideoInfo * best_info,
+    gboolean * at_least_one_alpha);
+
 static void gst_gl_stereo_mix_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_gl_stereo_mix_get_property (GObject * object, guint prop_id,
@@ -179,23 +130,24 @@ gst_gl_stereo_mix_class_init (GstGLStereoMixClass * klass)
           GST_TYPE_GL_STEREO_DOWNMIX_MODE_TYPE, DEFAULT_DOWNMIX,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_add_static_pad_template_with_gtype (element_class,
-      &src_factory, GST_TYPE_AGGREGATOR_PAD);
-  gst_element_class_add_static_pad_template_with_gtype (element_class,
-      &sink_factory, GST_TYPE_GL_STEREO_MIX_PAD);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_factory));
 
   agg_class->stop = gst_gl_stereo_mix_stop;
   agg_class->start = gst_gl_stereo_mix_start;
   agg_class->src_query = gst_gl_stereo_mix_src_query;
-  agg_class->negotiated_src_caps = _negotiated_caps;
 
   videoaggregator_class->aggregate_frames = gst_gl_stereo_mix_aggregate_frames;
   videoaggregator_class->update_caps = _update_caps;
+  videoaggregator_class->negotiated_caps = _negotiated_caps;
   videoaggregator_class->get_output_buffer =
       gst_gl_stereo_mix_get_output_buffer;
+  videoaggregator_class->find_best_format = gst_gl_stereo_mix_find_best_format;
+  videoaggregator_class->preserve_update_caps_result = TRUE;
 
-  base_mix_class->supported_gl_api =
-      GST_GL_API_GLES2 | GST_GL_API_OPENGL | GST_GL_API_OPENGL3;
+  base_mix_class->supported_gl_api = GST_GL_API_OPENGL | GST_GL_API_OPENGL3;
 }
 
 static void
@@ -290,8 +242,10 @@ gst_gl_stereo_mix_get_output_buffer (GstVideoAggregator * videoaggregator,
 gboolean
 gst_gl_stereo_mix_make_output (GstGLStereoMix * mix)
 {
+  guint i;
   GList *walk;
   gboolean res = FALSE;
+  guint array_index = 0;
   GstElement *element = GST_ELEMENT (mix);
   gboolean missing_buffer = FALSE;
 
@@ -299,23 +253,33 @@ gst_gl_stereo_mix_make_output (GstGLStereoMix * mix)
 
   GST_OBJECT_LOCK (mix);
   walk = element->sinkpads;
+
+  i = mix->frames->len;
+  g_ptr_array_set_size (mix->frames, element->numsinkpads);
+  for (; i < element->numsinkpads; i++)
+    mix->frames->pdata[i] = g_slice_new0 (GstGLStereoMixFrameData);
   while (walk) {
+    GstGLMixerPad *pad = GST_GL_MIXER_PAD (walk->data);
     GstVideoAggregatorPad *vaggpad = walk->data;
-    GstGLStereoMixPad *pad = walk->data;
+    GstGLStereoMixFrameData *frame;
 
     GST_LOG_OBJECT (mix, "Checking pad %" GST_PTR_FORMAT, vaggpad);
 
-    if (vaggpad->buffer != NULL) {
-      pad->current_buffer = vaggpad->buffer;
+    frame = g_ptr_array_index (mix->frames, array_index);
+    frame->base.pad = pad;
+    frame->buf = NULL;
 
-      GST_DEBUG_OBJECT (pad, "Got buffer %" GST_PTR_FORMAT,
-          pad->current_buffer);
+    walk = g_list_next (walk);
+
+    if (vaggpad->buffer != NULL) {
+      frame->buf = vaggpad->buffer;
+
+      GST_DEBUG_OBJECT (pad, "Got buffer %" GST_PTR_FORMAT, frame->buf);
     } else {
       GST_LOG_OBJECT (mix, "No buffer on pad %" GST_PTR_FORMAT, vaggpad);
-      pad->current_buffer = NULL;
       missing_buffer = TRUE;
     }
-    walk = g_list_next (walk);
+    ++array_index;
   }
   if (missing_buffer) {
     /* We're still waiting for a buffer to turn up on at least one input */
@@ -325,7 +289,7 @@ gst_gl_stereo_mix_make_output (GstGLStereoMix * mix)
   }
 
   /* Copy GL memory from each input frame to the output */
-  if (!gst_gl_stereo_mix_process_frames (mix)) {
+  if (!gst_gl_stereo_mix_process_frames (mix, mix->frames)) {
     GST_LOG_OBJECT (mix, "Failed to process frames to output");
     goto out;
   }
@@ -399,18 +363,41 @@ gst_gl_stereo_mix_set_property (GObject * object,
   }
 }
 
+static void
+_free_glmixer_frame_data (GstGLStereoMixFrameData * frame)
+{
+  if (frame == NULL)
+    return;
+  if (frame->buf)
+    gst_buffer_unref (frame->buf);
+  g_slice_free1 (sizeof (GstGLStereoMixFrameData), frame);
+}
+
 static gboolean
 gst_gl_stereo_mix_start (GstAggregator * agg)
 {
+  guint i;
   GstGLStereoMix *mix = GST_GL_STEREO_MIX (agg);
+  GstElement *element = GST_ELEMENT (agg);
 
   if (!GST_AGGREGATOR_CLASS (parent_class)->start (agg))
     return FALSE;
 
   GST_OBJECT_LOCK (mix);
+  mix->array_buffers = g_ptr_array_new_full (element->numsinkpads,
+      (GDestroyNotify) _free_glmixer_frame_data);
+  mix->frames = g_ptr_array_new_full (element->numsinkpads, NULL);
+
+  g_ptr_array_set_size (mix->array_buffers, element->numsinkpads);
+  g_ptr_array_set_size (mix->frames, element->numsinkpads);
+
+  for (i = 0; i < element->numsinkpads; i++)
+    mix->frames->pdata[i] = g_slice_new0 (GstGLStereoMixFrameData);
+
   mix->viewconvert = gst_gl_view_convert_new ();
   g_object_set (G_OBJECT (mix->viewconvert), "downmix-mode",
       mix->downmix_mode, NULL);
+
   GST_OBJECT_UNLOCK (mix);
 
   return TRUE;
@@ -424,6 +411,13 @@ gst_gl_stereo_mix_stop (GstAggregator * agg)
   if (!GST_AGGREGATOR_CLASS (parent_class)->stop (agg))
     return FALSE;
 
+  GST_OBJECT_LOCK (agg);
+  g_ptr_array_free (mix->frames, TRUE);
+  mix->frames = NULL;
+  g_ptr_array_free (mix->array_buffers, TRUE);
+  mix->array_buffers = NULL;
+  GST_OBJECT_UNLOCK (agg);
+
   if (mix->viewconvert) {
     gst_object_unref (mix->viewconvert);
     mix->viewconvert = NULL;
@@ -467,9 +461,154 @@ get_converted_caps (GstGLStereoMix * mix, GstCaps * caps)
   return result;
 }
 
-/* Return the possible output caps based on inputs and downstream prefs */
+/* Return the possible output caps we decided in find_best_format() */
 static GstCaps *
 _update_caps (GstVideoAggregator * vagg, GstCaps * caps)
+{
+  GstGLStereoMix *mix = GST_GL_STEREO_MIX (vagg);
+
+  return gst_caps_ref (mix->out_caps);
+}
+
+/* Called after videoaggregator fixates our caps */
+static gboolean
+_negotiated_caps (GstVideoAggregator * vagg, GstCaps * caps)
+{
+  GstGLStereoMix *mix = GST_GL_STEREO_MIX (vagg);
+
+  GST_LOG_OBJECT (mix, "Configured output caps %" GST_PTR_FORMAT, caps);
+
+  if (GST_VIDEO_AGGREGATOR_CLASS (parent_class)->negotiated_caps)
+    if (!GST_VIDEO_AGGREGATOR_CLASS (parent_class)->negotiated_caps (vagg,
+            caps))
+      return FALSE;
+
+  /* Update the glview_convert output */
+  if (!gst_video_info_from_caps (&mix->out_info, caps))
+    return FALSE;
+
+  /* We can configure the view_converter now */
+  gst_gl_view_convert_set_context (mix->viewconvert,
+      GST_GL_BASE_MIXER (mix)->context);
+  gst_gl_view_convert_set_format (mix->viewconvert, &mix->mix_info,
+      &mix->out_info);
+
+  return TRUE;
+
+}
+
+static gboolean
+gst_gl_stereo_mix_process_frames (GstGLStereoMix * mixer, GPtrArray * frames)
+{
+  GstVideoAggregator *vagg = GST_VIDEO_AGGREGATOR (mixer);
+  GstBuffer *converted_buffer, *inbuf;
+  GstVideoInfo *out_info = &vagg->info;
+  gint count = 0;
+#ifndef G_DISABLE_ASSERT
+  gint n;
+#endif
+  gint v, views;
+  gint valid_views = 0;
+
+  inbuf = gst_buffer_new ();
+  while (count < frames->len) {
+    GstGLStereoMixFrameData *frame;
+    GstMemory *in_mem;
+
+    frame = g_ptr_array_index (frames, count);
+    GST_LOG_OBJECT (mixer, "Handling frame %d", count);
+
+    if (!frame) {
+      GST_DEBUG ("skipping texture, null frame");
+      count++;
+      continue;
+    }
+
+    in_mem = gst_buffer_get_memory (frame->buf, 0);
+
+    GST_LOG_OBJECT (mixer,
+        "Appending memory %" GST_PTR_FORMAT " to intermediate buffer", in_mem);
+    /* Appending the memory to a 2nd buffer locks it
+     * exclusive a 2nd time, which will mark it for
+     * copy-on-write. The ref will keep the memory
+     * alive but we add a parent_buffer_meta to also
+     * prevent the input buffer from returning to any buffer
+     * pool it might belong to
+     */
+    gst_buffer_append_memory (inbuf, in_mem);
+    /* Use parent buffer meta to keep input buffer alive */
+    gst_buffer_add_parent_buffer_meta (inbuf, frame->buf);
+
+    count++;
+    valid_views++;
+  }
+
+  if (mixer->mix_info.views != valid_views) {
+    GST_WARNING_OBJECT (mixer, "Not enough input views to process");
+    return FALSE;
+  }
+
+  if (GST_VIDEO_INFO_MULTIVIEW_MODE (out_info) ==
+      GST_VIDEO_MULTIVIEW_MODE_SEPARATED)
+    views = out_info->views;
+  else
+    views = 1;
+
+  if (gst_gl_view_convert_submit_input_buffer (mixer->viewconvert,
+          FALSE, inbuf) != GST_FLOW_OK)
+    return FALSE;
+
+  /* Clear any existing buffers, just in case */
+  gst_buffer_replace (&mixer->primary_out, NULL);
+  gst_buffer_replace (&mixer->auxilliary_out, NULL);
+
+  if (gst_gl_view_convert_get_output (mixer->viewconvert,
+          &mixer->primary_out) != GST_FLOW_OK)
+    return FALSE;
+
+  if (GST_VIDEO_INFO_MULTIVIEW_MODE (out_info) ==
+      GST_VIDEO_MULTIVIEW_MODE_FRAME_BY_FRAME) {
+    if (gst_gl_view_convert_get_output (mixer->viewconvert,
+            &mixer->auxilliary_out) != GST_FLOW_OK)
+      return FALSE;
+  }
+
+  if (mixer->primary_out == NULL)
+    return FALSE;
+
+  converted_buffer = mixer->primary_out;
+
+#ifndef G_DISABLE_ASSERT
+  n = gst_buffer_n_memory (converted_buffer);
+  g_assert (n == GST_VIDEO_INFO_N_PLANES (out_info) * views);
+#endif
+
+  for (v = 0; v < views; v++) {
+    gst_buffer_add_video_meta_full (converted_buffer, v,
+        GST_VIDEO_INFO_FORMAT (out_info),
+        GST_VIDEO_INFO_WIDTH (out_info),
+        GST_VIDEO_INFO_HEIGHT (out_info),
+        GST_VIDEO_INFO_N_PLANES (out_info), out_info->offset, out_info->stride);
+    if (mixer->auxilliary_out) {
+      gst_buffer_add_video_meta_full (mixer->auxilliary_out, v,
+          GST_VIDEO_INFO_FORMAT (out_info),
+          GST_VIDEO_INFO_WIDTH (out_info),
+          GST_VIDEO_INFO_HEIGHT (out_info),
+          GST_VIDEO_INFO_N_PLANES (out_info), out_info->offset,
+          out_info->stride);
+    }
+  }
+
+  return TRUE;
+}
+
+/* Iterate the input sink pads, and choose the blend format
+ * we will generate before output conversion, which is RGBA
+ * at some suitable size */
+static void
+gst_gl_stereo_mix_find_best_format (GstVideoAggregator * vagg,
+    GstCaps * downstream_caps, GstVideoInfo * best_info,
+    gboolean * at_least_one_alpha)
 {
   GstGLStereoMix *mix = GST_GL_STEREO_MIX (vagg);
   GList *l;
@@ -478,7 +617,10 @@ _update_caps (GstVideoAggregator * vagg, GstCaps * caps)
   gint best_fps_n = 0, best_fps_d = 1;
   GstVideoInfo *mix_info;
   GstCaps *blend_caps, *tmp_caps;
-  GstCaps *out_caps;
+
+  /* We'll deal with alpha internally, so just tell aggregator to
+   * be quiet */
+  *at_least_one_alpha = FALSE;
 
   GST_OBJECT_LOCK (vagg);
 
@@ -552,147 +694,15 @@ _update_caps (GstVideoAggregator * vagg, GstCaps * caps)
   tmp_caps = get_converted_caps (GST_GL_STEREO_MIX (vagg), blend_caps);
   gst_caps_unref (blend_caps);
 
-  out_caps = gst_caps_intersect (caps, tmp_caps);
+  if (mix->out_caps)
+    gst_caps_unref (mix->out_caps);
+
+  mix->out_caps = gst_caps_intersect (downstream_caps, tmp_caps);
   gst_caps_unref (tmp_caps);
 
-  GST_DEBUG_OBJECT (vagg, "Possible output caps %" GST_PTR_FORMAT, out_caps);
-
-  return out_caps;
-}
-
-/* Called after videoaggregator fixates our caps */
-static gboolean
-_negotiated_caps (GstAggregator * agg, GstCaps * caps)
-{
-  GstVideoAggregator *vagg = GST_VIDEO_AGGREGATOR (agg);
-  GstGLStereoMix *mix = GST_GL_STEREO_MIX (vagg);
-  GstCaps *in_caps;
-
-  GST_LOG_OBJECT (mix, "Configured output caps %" GST_PTR_FORMAT, caps);
-
-  if (GST_AGGREGATOR_CLASS (parent_class)->negotiated_src_caps)
-    if (!GST_AGGREGATOR_CLASS (parent_class)->negotiated_src_caps (agg, caps))
-      return FALSE;
-
-  /* Update the glview_convert output */
-
-  /* We can configure the view_converter now */
-  gst_gl_view_convert_set_context (mix->viewconvert,
-      GST_GL_BASE_MIXER (mix)->context);
-
-  in_caps = gst_video_info_to_caps (&mix->mix_info);
-  gst_caps_set_features (in_caps, 0,
-      gst_caps_features_from_string (GST_CAPS_FEATURE_MEMORY_GL_MEMORY));
-  gst_caps_set_simple (in_caps, "texture-target", G_TYPE_STRING,
-      GST_GL_TEXTURE_TARGET_2D_STR, NULL);
-
-  gst_gl_view_convert_set_caps (mix->viewconvert, in_caps, caps);
-
-  return TRUE;
-}
-
-/* called with the object lock held */
-static gboolean
-gst_gl_stereo_mix_process_frames (GstGLStereoMix * mixer)
-{
-  GstVideoAggregator *vagg = GST_VIDEO_AGGREGATOR (mixer);
-  GstBuffer *converted_buffer, *inbuf;
-  GstVideoInfo *out_info = &vagg->info;
-#ifndef G_DISABLE_ASSERT
-  gint n;
-#endif
-  gint v, views;
-  gint valid_views = 0;
-  GList *walk;
-
-  inbuf = gst_buffer_new ();
-  walk = GST_ELEMENT (mixer)->sinkpads;
-  while (walk) {
-    GstGLStereoMixPad *pad = walk->data;
-    GstMemory *in_mem;
-
-    GST_LOG_OBJECT (mixer, "Handling frame %d", valid_views);
-
-    if (!pad || !pad->current_buffer) {
-      GST_DEBUG ("skipping texture, null frame");
-      walk = g_list_next (walk);
-      continue;
-    }
-
-    in_mem = gst_buffer_get_memory (pad->current_buffer, 0);
-
-    GST_LOG_OBJECT (mixer,
-        "Appending memory %" GST_PTR_FORMAT " to intermediate buffer", in_mem);
-    /* Appending the memory to a 2nd buffer locks it
-     * exclusive a 2nd time, which will mark it for
-     * copy-on-write. The ref will keep the memory
-     * alive but we add a parent_buffer_meta to also
-     * prevent the input buffer from returning to any buffer
-     * pool it might belong to
-     */
-    gst_buffer_append_memory (inbuf, in_mem);
-    /* Use parent buffer meta to keep input buffer alive */
-    gst_buffer_add_parent_buffer_meta (inbuf, pad->current_buffer);
-
-    valid_views++;
-    walk = g_list_next (walk);
-  }
-
-  if (mixer->mix_info.views != valid_views) {
-    GST_WARNING_OBJECT (mixer, "Not enough input views to process");
-    return FALSE;
-  }
-
-  if (GST_VIDEO_INFO_MULTIVIEW_MODE (out_info) ==
-      GST_VIDEO_MULTIVIEW_MODE_SEPARATED)
-    views = out_info->views;
-  else
-    views = 1;
-
-  if (gst_gl_view_convert_submit_input_buffer (mixer->viewconvert,
-          FALSE, inbuf) != GST_FLOW_OK)
-    return FALSE;
-
-  /* Clear any existing buffers, just in case */
-  gst_buffer_replace (&mixer->primary_out, NULL);
-  gst_buffer_replace (&mixer->auxilliary_out, NULL);
-
-  if (gst_gl_view_convert_get_output (mixer->viewconvert,
-          &mixer->primary_out) != GST_FLOW_OK)
-    return FALSE;
-
-  if (GST_VIDEO_INFO_MULTIVIEW_MODE (out_info) ==
-      GST_VIDEO_MULTIVIEW_MODE_FRAME_BY_FRAME) {
-    if (gst_gl_view_convert_get_output (mixer->viewconvert,
-            &mixer->auxilliary_out) != GST_FLOW_OK)
-      return FALSE;
-  }
-
-  if (mixer->primary_out == NULL)
-    return FALSE;
-
-  converted_buffer = mixer->primary_out;
-
-#ifndef G_DISABLE_ASSERT
-  n = gst_buffer_n_memory (converted_buffer);
-  g_assert (n == GST_VIDEO_INFO_N_PLANES (out_info) * views);
-#endif
-
-  for (v = 0; v < views; v++) {
-    gst_buffer_add_video_meta_full (converted_buffer, v,
-        GST_VIDEO_INFO_FORMAT (out_info),
-        GST_VIDEO_INFO_WIDTH (out_info),
-        GST_VIDEO_INFO_HEIGHT (out_info),
-        GST_VIDEO_INFO_N_PLANES (out_info), out_info->offset, out_info->stride);
-    if (mixer->auxilliary_out) {
-      gst_buffer_add_video_meta_full (mixer->auxilliary_out, v,
-          GST_VIDEO_INFO_FORMAT (out_info),
-          GST_VIDEO_INFO_WIDTH (out_info),
-          GST_VIDEO_INFO_HEIGHT (out_info),
-          GST_VIDEO_INFO_N_PLANES (out_info), out_info->offset,
-          out_info->stride);
-    }
-  }
-
-  return TRUE;
+  GST_DEBUG_OBJECT (vagg, "Possible output caps %" GST_PTR_FORMAT,
+      mix->out_caps);
+  /* Tell videoaggregator our preferred size. Actual info gets
+   * overridden during caps nego */
+  *best_info = *mix_info;
 }
