@@ -3,7 +3,7 @@
  * Copyright (C) 2005 Thomas Vander Stichele <thomas@apestaart.org>
  * Copyright (C) 2005 Ronald S. Bultje <rbultje@ronald.bitfreak.net>
  * Copyright (C) 2008 Michael Sheldon <mike@mikeasoft.com>
- *
+ * 
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
@@ -46,7 +46,14 @@
 #ifndef __GST_TEMPLATE_MATCH_H__
 #define __GST_TEMPLATE_MATCH_H__
 
-#include <gst/opencv/gstopencvvideofilter.h>
+#include <gst/gst.h>
+
+#ifdef HAVE_HIGHGUI_H
+#include <highgui.h>            // includes highGUI definitions
+#endif
+#ifdef HAVE_OPENCV2_HIGHGUI_HIGHGUI_C_H
+#include <opencv2/highgui/highgui_c.h>            // includes highGUI definitions
+#endif
 
 G_BEGIN_DECLS
 /* #defines don't like whitespacey bits */
@@ -65,20 +72,21 @@ typedef struct _GstTemplateMatchClass GstTemplateMatchClass;
 
 struct _GstTemplateMatch
 {
-  GstOpencvVideoFilter element;
+  GstElement element;
+
+  GstPad *sinkpad, *srcpad;
 
   gint method;
   gboolean display;
 
   gchar *templ;
 
-  cv::Mat cvTemplateImage, cvDistImage;
-  gboolean reload_dist_image;
+  IplImage *cvImage, *cvGray, *cvTemplateImage, *cvDistImage;
 };
 
 struct _GstTemplateMatchClass
 {
-  GstOpencvVideoFilterClass parent_class;
+  GstElementClass parent_class;
 };
 
 GType gst_template_match_get_type (void);
