@@ -21,15 +21,15 @@
 
 /**
  * SECTION:element-sbdec
- * @title: sbdec
  *
  * This element decodes a Bluetooth SBC audio streams to raw integer PCM audio
  *
- * ## Example pipelines
+ * <refsect2>
+ * <title>Example pipelines</title>
  * |[
  * gst-launch-1.0 -v filesrc location=audio.sbc ! sbcparse ! sbcdec ! audioconvert ! audioresample ! autoaudiosink
  * ]| Decode a raw SBC file.
- *
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -159,14 +159,13 @@ gst_sbc_dec_set_format (GstAudioDecoder * audio_dec, GstCaps * caps)
     return FALSE;
 
   if (strcmp (channel_mode, "mono") == 0) {
-    dec->frame_len = 4 + (subbands * 1) / 2 + ((blocks * 1 * bitpool) + 7) / 8;
+    dec->frame_len = 4 + (subbands * 1) / 2 + (blocks * 1 * bitpool) / 8;
   } else if (strcmp (channel_mode, "dual") == 0) {
-    dec->frame_len = 4 + (subbands * 2) / 2 + ((blocks * 2 * bitpool) + 7) / 8;
+    dec->frame_len = 4 + (subbands * 2) / 2 + (blocks * 2 * bitpool) / 8;
   } else if (strcmp (channel_mode, "stereo") == 0) {
-    dec->frame_len = 4 + (subbands * 2) / 2 + ((blocks * bitpool) + 7) / 8;
+    dec->frame_len = 4 + (subbands * 2) / 2 + (blocks * bitpool) / 8;
   } else if (strcmp (channel_mode, "joint") == 0) {
-    dec->frame_len =
-        4 + (subbands * 2) / 2 + ((subbands + blocks * bitpool) + 7) / 8;
+    dec->frame_len = 4 + (subbands * 2) / 2 + (subbands + blocks * bitpool) / 8;
   } else {
     return FALSE;
   }
@@ -220,10 +219,11 @@ gst_sbc_dec_class_init (GstSbcDecClass * klass)
   audio_decoder_class->handle_frame =
       GST_DEBUG_FUNCPTR (gst_sbc_dec_handle_frame);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &sbc_dec_sink_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &sbc_dec_src_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sbc_dec_sink_factory));
+
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sbc_dec_src_factory));
 
   gst_element_class_set_static_metadata (element_class,
       "Bluetooth SBC audio decoder", "Codec/Decoder/Audio",
