@@ -34,7 +34,7 @@
 GST_DEBUG_CATEGORY_EXTERN (gstwayland_debug);
 #define GST_CAT_DEFAULT gstwayland_debug
 
-G_DEFINE_TYPE (GstWlShmAllocator, gst_wl_shm_allocator, GST_TYPE_FD_ALLOCATOR);
+G_DEFINE_TYPE (GstWlShmAllocator, gst_wl_shm_allocator, GST_TYPE_ALLOCATOR);
 
 static GstMemory *
 gst_wl_shm_allocator_alloc (GstAllocator * allocator, gsize size,
@@ -44,8 +44,8 @@ gst_wl_shm_allocator_alloc (GstAllocator * allocator, gsize size,
   char filename[1024];
   static int init = 0;
   int fd;
-  GstMemory *mem;
-  GstMapInfo info;
+  gpointer data;
+  GstWlShmMemory *mem;
 
   /* TODO: make use of the allocation params, if necessary */
 
@@ -65,28 +65,45 @@ gst_wl_shm_allocator_alloc (GstAllocator * allocator, gsize size,
     return NULL;
   }
 
-  mem = gst_fd_allocator_alloc (allocator, fd, size,
-      GST_FD_MEMORY_FLAG_KEEP_MAPPED);
-  if (G_UNLIKELY (!mem)) {
-    GST_ERROR_OBJECT (self, "GstFdMemory allocation failed");
+  data = mmap (NULL, size, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
+  if (data == MAP_FAILED) {
+    GST_ERROR_OBJECT (self, "mmap failed: %s", strerror (errno));
     close (fd);
     return NULL;
   }
 
-  /* we need to map the memory in order to unlink the file without losing it */
-  if (!gst_memory_map (mem, &info, GST_MAP_READWRITE)) {
-    GST_ERROR_OBJECT (self, "GstFdMemory map failed");
-    close (fd);
-    return NULL;
-  }
-
-  /* unmap will not really munmap(), we just
-   * need it to release the miniobject lock */
-  gst_memory_unmap (mem, &info);
-
   unlink (filename);
 
-  return mem;
+  mem = g_slice_new0 (GstWlShmMemory);
+  gst_memory_init ((GstMemory *) mem, GST_MEMORY_FLAG_NO_SHARE, allocator, NULL,
+      size, 0, 0, size);
+  mem->data = data;
+  mem->fd = fd;
+
+  return (GstMemory *) mem;
+}
+
+static void
+gst_wl_shm_allocator_free (GstAllocator * allocator, GstMemory * memory)
+{
+  GstWlShmMemory *shm_mem = (GstWlShmMemory *) memory;
+
+  if (shm_mem->fd != -1)
+    close (shm_mem->fd);
+  munmap (shm_mem->data, memory->maxsize);
+
+  g_slice_free (GstWlShmMemory, shm_mem);
+}
+
+static gpointer
+gst_wl_shm_mem_map (GstMemory * mem, gsize maxsize, GstMapFlags flags)
+{
+  return ((GstWlShmMemory *) mem)->data;
+}
+
+static void
+gst_wl_shm_mem_unmap (GstMemory * mem)
+{
 }
 
 static void
@@ -95,26 +112,24 @@ gst_wl_shm_allocator_class_init (GstWlShmAllocatorClass * klass)
   GstAllocatorClass *alloc_class = (GstAllocatorClass *) klass;
 
   alloc_class->alloc = GST_DEBUG_FUNCPTR (gst_wl_shm_allocator_alloc);
+  alloc_class->free = GST_DEBUG_FUNCPTR (gst_wl_shm_allocator_free);
 }
 
 static void
 gst_wl_shm_allocator_init (GstWlShmAllocator * self)
 {
-  GstAllocator *alloc = GST_ALLOCATOR_CAST (self);
+  self->parent_instance.mem_type = GST_ALLOCATOR_WL_SHM;
+  self->parent_instance.mem_map = gst_wl_shm_mem_map;
+  self->parent_instance.mem_unmap = gst_wl_shm_mem_unmap;
 
-  alloc->mem_type = GST_ALLOCATOR_WL_SHM;
-
-  GST_OBJECT_FLAG_UNSET (self, GST_ALLOCATOR_FLAG_CUSTOM_ALLOC);
+  GST_OBJECT_FLAG_SET (self, GST_ALLOCATOR_FLAG_CUSTOM_ALLOC);
 }
 
 void
 gst_wl_shm_allocator_register (void)
 {
-  GstAllocator *alloc;
-
-  alloc = g_object_new (GST_TYPE_WL_SHM_ALLOCATOR, NULL);
-  gst_object_ref_sink (alloc);
-  gst_allocator_register (GST_ALLOCATOR_WL_SHM, alloc);
+  gst_allocator_register (GST_ALLOCATOR_WL_SHM,
+      g_object_new (GST_TYPE_WL_SHM_ALLOCATOR, NULL));
 }
 
 GstAllocator *
@@ -129,102 +144,37 @@ gst_is_wl_shm_memory (GstMemory * mem)
   return gst_memory_is_type (mem, GST_ALLOCATOR_WL_SHM);
 }
 
-/* Copied from gst_v4l2_object_extrapolate_stride() */
-static gint
-gst_wl_shm_extrapolate_stride (const GstVideoFormatInfo * finfo, gint plane,
-    gint stride)
-{
-  gint estride;
-
-  switch (finfo->format) {
-    case GST_VIDEO_FORMAT_NV12:
-    case GST_VIDEO_FORMAT_NV12_64Z32:
-    case GST_VIDEO_FORMAT_NV21:
-    case GST_VIDEO_FORMAT_NV16:
-    case GST_VIDEO_FORMAT_NV61:
-    case GST_VIDEO_FORMAT_NV24:
-      estride = (plane == 0 ? 1 : 2) *
-          GST_VIDEO_FORMAT_INFO_SCALE_WIDTH (finfo, plane, stride);
-      break;
-    default:
-      estride = GST_VIDEO_FORMAT_INFO_SCALE_WIDTH (finfo, plane, stride);
-      break;
-  }
-
-  return estride;
-}
-
-static gboolean
-gst_wl_shm_validate_video_info (const GstVideoInfo * vinfo)
-{
-  gint height = GST_VIDEO_INFO_HEIGHT (vinfo);
-  gint base_stride = GST_VIDEO_INFO_PLANE_STRIDE (vinfo, 0);
-  gsize base_offs = GST_VIDEO_INFO_PLANE_OFFSET (vinfo, 0);
-  gint i;
-  gsize offs = 0;
-
-  for (i = 0; i < GST_VIDEO_INFO_N_PLANES (vinfo); i++) {
-    guint32 estride;
-
-    /* Overwrite the video info's stride and offset using the pitch calculcated
-     * by the kms driver. */
-    estride = gst_wl_shm_extrapolate_stride (vinfo->finfo, i, base_stride);
-
-    if (estride != GST_VIDEO_INFO_PLANE_STRIDE (vinfo, i))
-      return FALSE;
-
-    if (GST_VIDEO_INFO_PLANE_OFFSET (vinfo, i) - base_offs != offs)
-      return FALSE;
-
-    /* Note that we cannot negotiate special padding betweem each planes,
-     * hence using the display height here. */
-    offs +=
-        estride * GST_VIDEO_FORMAT_INFO_SCALE_HEIGHT (vinfo->finfo, i, height);
-  }
-
-  if (vinfo->size < offs)
-    return FALSE;
-
-  return TRUE;
-}
-
 struct wl_buffer *
 gst_wl_shm_memory_construct_wl_buffer (GstMemory * mem, GstWlDisplay * display,
     const GstVideoInfo * info)
 {
+  GstWlShmMemory *shm_mem = (GstWlShmMemory *) mem;
   gint width, height, stride;
-  gsize offset, size, memsize, maxsize;
+  gsize size;
   enum wl_shm_format format;
   struct wl_shm_pool *wl_pool;
   struct wl_buffer *wbuffer;
 
-  if (!gst_wl_shm_validate_video_info (info)) {
-    GST_DEBUG_OBJECT (display, "Unsupported strides and offsets.");
-    return NULL;
-  }
-
   width = GST_VIDEO_INFO_WIDTH (info);
   height = GST_VIDEO_INFO_HEIGHT (info);
   stride = GST_VIDEO_INFO_PLANE_STRIDE (info, 0);
   size = GST_VIDEO_INFO_SIZE (info);
   format = gst_video_format_to_wl_shm_format (GST_VIDEO_INFO_FORMAT (info));
 
-  memsize = gst_memory_get_sizes (mem, &offset, &maxsize);
-  offset += GST_VIDEO_INFO_PLANE_OFFSET (info, 0);
+  g_return_val_if_fail (gst_is_wl_shm_memory (mem), NULL);
+  g_return_val_if_fail (size <= mem->size, NULL);
+  g_return_val_if_fail (shm_mem->fd != -1, NULL);
 
-  g_return_val_if_fail (gst_is_fd_memory (mem), NULL);
-  g_return_val_if_fail (size <= memsize, NULL);
-  g_return_val_if_fail (gst_wl_display_check_format_for_shm (display,
-          GST_VIDEO_INFO_FORMAT (info)), NULL);
-
-  GST_DEBUG_OBJECT (display, "Creating wl_buffer from SHM of size %"
+  GST_DEBUG_OBJECT (mem->allocator, "Creating wl_buffer of size %"
       G_GSSIZE_FORMAT " (%d x %d, stride %d), format %s", size, width, height,
       stride, gst_wl_shm_format_to_string (format));
 
-  wl_pool = wl_shm_create_pool (display->shm, gst_fd_memory_get_fd (mem),
-      memsize);
-  wbuffer = wl_shm_pool_create_buffer (wl_pool, offset, width, height, stride,
+  wl_pool = wl_shm_create_pool (display->shm, shm_mem->fd, mem->size);
+  wbuffer = wl_shm_pool_create_buffer (wl_pool, 0, width, height, stride,
       format);
+
+  close (shm_mem->fd);
+  shm_mem->fd = -1;
   wl_shm_pool_destroy (wl_pool);
 
   return wbuffer;
