@@ -21,40 +21,46 @@
  */
 /**
  * SECTION: gstaggregator
- * @title: GstAggregator
  * @short_description: manages a set of pads with the purpose of
  * aggregating their buffers.
  * @see_also: gstcollectpads for historical reasons.
  *
  * Manages a set of pads with the purpose of aggregating their buffers.
  * Control is given to the subclass when all pads have data.
- *
- *  * Base class for mixers and muxers. Subclasses should at least implement
+ * <itemizedlist>
+ *  <listitem><para>
+ *    Base class for mixers and muxers. Subclasses should at least implement
  *    the #GstAggregatorClass.aggregate() virtual method.
- *
- *  * When data is queued on all pads, tha aggregate vmethod is called.
- *
- *  * One can peek at the data on any given GstAggregatorPad with the
+ *  </para></listitem>
+ *  <listitem><para>
+ *    When data is queued on all pads, tha aggregate vmethod is called.
+ *  </para></listitem>
+ *  <listitem><para>
+ *    One can peek at the data on any given GstAggregatorPad with the
  *    gst_aggregator_pad_get_buffer () method, and take ownership of it
  *    with the gst_aggregator_pad_steal_buffer () method. When a buffer
  *    has been taken with steal_buffer (), a new buffer can be queued
  *    on that pad.
- *
- *  * If the subclass wishes to push a buffer downstream in its aggregate
+ *  </para></listitem>
+ *  <listitem><para>
+ *    If the subclass wishes to push a buffer downstream in its aggregate
  *    implementation, it should do so through the
  *    gst_aggregator_finish_buffer () method. This method will take care
  *    of sending and ordering mandatory events such as stream start, caps
  *    and segment.
- *
- *  * Same goes for EOS events, which should not be pushed directly by the
+ *  </para></listitem>
+ *  <listitem><para>
+ *    Same goes for EOS events, which should not be pushed directly by the
  *    subclass, it should instead return GST_FLOW_EOS in its aggregate
  *    implementation.
- *
- *  * Note that the aggregator logic regarding gap event handling is to turn
+ *  </para></listitem>
+ *  <listitem><para>
+ *    Note that the aggregator logic regarding gap event handling is to turn
  *    these into gap buffers with matching PTS and duration. It will also
  *    flag these buffers with GST_BUFFER_FLAG_GAP and GST_BUFFER_FLAG_DROPPABLE
  *    to ease their identification and subsequent processing.
- *
+ *  </para></listitem>
+ * </itemizedlist>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -209,8 +215,6 @@ struct _GstAggregatorPadPrivate
   gboolean pending_flush_stop;
   gboolean pending_eos;
 
-  gboolean first_buffer;
-
   GQueue buffers;
   guint num_buffers;
   GstClockTime head_position;
@@ -264,7 +268,7 @@ static GstElementClass *aggregator_parent_class = NULL;
 
 struct _GstAggregatorPrivate
 {
-  gint max_padserial;
+  gint padcount;
 
   /* Our state is >= PAUSED */
   gboolean running;             /* protected by src_lock */
@@ -284,7 +288,7 @@ struct _GstAggregatorPrivate
   gboolean peer_latency_live;   /* protected by src_lock */
   GstClockTime peer_latency_min;        /* protected by src_lock */
   GstClockTime peer_latency_max;        /* protected by src_lock */
-  gboolean has_peer_latency;    /* protected by src_lock */
+  gboolean has_peer_latency;
 
   GstClockTime sub_latency_min; /* protected by src_lock */
   GstClockTime sub_latency_max; /* protected by src_lock */
@@ -294,7 +298,7 @@ struct _GstAggregatorPrivate
   GMutex src_lock;
   GCond src_cond;
 
-  gboolean first_buffer;        /* protected by object lock */
+  gboolean first_buffer;
   GstAggregatorStartTimeSelection start_time_selection;
   GstClockTime start_time;
 
@@ -307,7 +311,6 @@ typedef struct
   GstEvent *event;
   gboolean result;
   gboolean flush;
-  gboolean only_to_active_pads;
 
   gboolean one_actually_seeked;
 } EventData;
@@ -418,7 +421,6 @@ gst_aggregator_check_pads_ready (GstAggregator * self)
 {
   GstAggregatorPad *pad;
   GList *l, *sinkpads;
-  gboolean have_data = TRUE;
 
   GST_LOG_OBJECT (self, "checking pads");
 
@@ -433,29 +435,21 @@ gst_aggregator_check_pads_ready (GstAggregator * self)
 
     PAD_LOCK (pad);
 
-    if (gst_aggregator_pad_queue_is_empty (pad)) {
-      if (!pad->priv->eos) {
-        have_data = FALSE;
-
-        /* If not live we need data on all pads, so leave the loop */
-        if (!self->priv->peer_latency_live) {
-          PAD_UNLOCK (pad);
-          goto pad_not_ready;
-        }
-      }
-    } else if (self->priv->peer_latency_live) {
-      /* In live mode, having a single pad with buffers is enough to
-       * generate a start time from it. In non-live mode all pads need
-       * to have a buffer
-       */
+    /* In live mode, having a single pad with buffers is enough to
+     * generate a start time from it. In non-live mode all pads need
+     * to have a buffer
+     */
+    if (self->priv->peer_latency_live &&
+        !gst_aggregator_pad_queue_is_empty (pad))
       self->priv->first_buffer = FALSE;
+
+    if (gst_aggregator_pad_queue_is_empty (pad) && !pad->priv->eos) {
+      PAD_UNLOCK (pad);
+      goto pad_not_ready;
     }
-
     PAD_UNLOCK (pad);
-  }
 
-  if (!have_data)
-    goto pad_not_ready;
+  }
 
   self->priv->first_buffer = FALSE;
 
@@ -652,7 +646,6 @@ gst_aggregator_wait_and_check (GstAggregator * self, gboolean * timeout)
    * and if a pad does not have a buffer in time we ignore
    * that pad.
    */
-  GST_OBJECT_LOCK (self);
   if (!GST_CLOCK_TIME_IS_VALID (latency) ||
       !GST_IS_CLOCK (GST_ELEMENT_CLOCK (self)) ||
       !GST_CLOCK_TIME_IS_VALID (start) ||
@@ -663,7 +656,6 @@ gst_aggregator_wait_and_check (GstAggregator * self, gboolean * timeout)
      * then check if we're ready now. If we return FALSE,
      * we will be directly called again.
      */
-    GST_OBJECT_UNLOCK (self);
     SRC_WAIT (self);
   } else {
     GstClockTime base_time, time;
@@ -674,8 +666,11 @@ gst_aggregator_wait_and_check (GstAggregator * self, gboolean * timeout)
     GST_DEBUG_OBJECT (self, "got subclass start time: %" GST_TIME_FORMAT,
         GST_TIME_ARGS (start));
 
+    GST_OBJECT_LOCK (self);
     base_time = GST_ELEMENT_CAST (self)->base_time;
-    clock = gst_object_ref (GST_ELEMENT_CLOCK (self));
+    clock = GST_ELEMENT_CLOCK (self);
+    if (clock)
+      gst_object_ref (clock);
     GST_OBJECT_UNLOCK (self);
 
     time = base_time + start;
@@ -685,7 +680,7 @@ gst_aggregator_wait_and_check (GstAggregator * self, gboolean * timeout)
         GST_TIME_FORMAT " (base %" GST_TIME_FORMAT " start %" GST_TIME_FORMAT
         " latency %" GST_TIME_FORMAT " current %" GST_TIME_FORMAT ")",
         GST_TIME_ARGS (time),
-        GST_TIME_ARGS (base_time),
+        GST_TIME_ARGS (GST_ELEMENT_CAST (self)->base_time),
         GST_TIME_ARGS (start), GST_TIME_ARGS (latency),
         GST_TIME_ARGS (gst_clock_get_time (clock)));
 
@@ -703,8 +698,9 @@ gst_aggregator_wait_and_check (GstAggregator * self, gboolean * timeout)
     }
 
     GST_DEBUG_OBJECT (self,
-        "clock returned %d (jitter: %" GST_STIME_FORMAT ")",
-        status, GST_STIME_ARGS (jitter));
+        "clock returned %d (jitter: %s%" GST_TIME_FORMAT ")",
+        status, (jitter < 0 ? "-" : " "),
+        GST_TIME_ARGS ((jitter < 0 ? -jitter : jitter)));
 
     /* we timed out */
     if (status == GST_CLOCK_OK || status == GST_CLOCK_EARLY) {
@@ -862,6 +858,7 @@ gst_aggregator_start (GstAggregator * self)
   GstAggregatorClass *klass;
   gboolean result;
 
+  self->priv->running = TRUE;
   self->priv->send_stream_start = TRUE;
   self->priv->send_segment = TRUE;
   self->priv->send_eos = TRUE;
@@ -1089,8 +1086,6 @@ gst_aggregator_default_sink_event (GstAggregator * self,
         GST_OBJECT_UNLOCK (self);
       }
 
-      aggpad->priv->first_buffer = TRUE;
-
       /* We never forward the event */
       goto eat;
     }
@@ -1308,62 +1303,42 @@ gst_aggregator_release_pad (GstElement * element, GstPad * pad)
   SRC_UNLOCK (self);
 }
 
-static GstAggregatorPad *
-gst_aggregator_default_create_new_pad (GstAggregator * self,
-    GstPadTemplate * templ, const gchar * req_name, const GstCaps * caps)
-{
-  GstAggregatorPad *agg_pad;
-  GstAggregatorPrivate *priv = self->priv;
-  gint serial = 0;
-  gchar *name = NULL;
-
-  if (templ->direction != GST_PAD_SINK ||
-      g_strcmp0 (templ->name_template, "sink_%u") != 0)
-    goto not_sink;
-
-  GST_OBJECT_LOCK (self);
-  if (req_name == NULL || strlen (req_name) < 6
-      || !g_str_has_prefix (req_name, "sink_")) {
-    /* no name given when requesting the pad, use next available int */
-    serial = ++priv->max_padserial;
-  } else {
-    /* parse serial number from requested padname */
-    serial = g_ascii_strtoull (&req_name[5], NULL, 10);
-    if (serial > priv->max_padserial)
-      priv->max_padserial = serial;
-  }
-
-  name = g_strdup_printf ("sink_%u", serial);
-  agg_pad = g_object_new (GST_AGGREGATOR_GET_CLASS (self)->sinkpads_type,
-      "name", name, "direction", GST_PAD_SINK, "template", templ, NULL);
-  g_free (name);
-
-  GST_OBJECT_UNLOCK (self);
-
-  return agg_pad;
-
-  /* errors */
-not_sink:
-  {
-    GST_WARNING_OBJECT (self, "request new pad that is not a SINK pad\n");
-    return NULL;
-  }
-}
-
 static GstPad *
 gst_aggregator_request_new_pad (GstElement * element,
     GstPadTemplate * templ, const gchar * req_name, const GstCaps * caps)
 {
   GstAggregator *self;
   GstAggregatorPad *agg_pad;
-  GstAggregatorClass *klass = GST_AGGREGATOR_GET_CLASS (element);
+
+  GstElementClass *klass = GST_ELEMENT_GET_CLASS (element);
   GstAggregatorPrivate *priv = GST_AGGREGATOR (element)->priv;
 
   self = GST_AGGREGATOR (element);
 
-  agg_pad = klass->create_new_pad (self, templ, req_name, caps);
-  if (!agg_pad) {
-    GST_ERROR_OBJECT (element, "Couldn't create new pad");
+  if (templ == gst_element_class_get_pad_template (klass, "sink_%u")) {
+    gint serial = 0;
+    gchar *name = NULL;
+
+    GST_OBJECT_LOCK (element);
+    if (req_name == NULL || strlen (req_name) < 6
+        || !g_str_has_prefix (req_name, "sink_")) {
+      /* no name given when requesting the pad, use next available int */
+      priv->padcount++;
+    } else {
+      /* parse serial number from requested padname */
+      serial = g_ascii_strtoull (&req_name[5], NULL, 10);
+      if (serial >= priv->padcount)
+        priv->padcount = serial;
+    }
+
+    name = g_strdup_printf ("sink_%u", priv->padcount);
+    agg_pad = g_object_new (GST_AGGREGATOR_GET_CLASS (self)->sinkpads_type,
+        "name", name, "direction", GST_PAD_SINK, "template", templ, NULL);
+    g_free (name);
+
+    GST_OBJECT_UNLOCK (element);
+
+  } else {
     return NULL;
   }
 
@@ -1426,6 +1401,18 @@ gst_aggregator_query_latency_unlocked (GstAggregator * self, GstQuery * query)
   else
     max = GST_CLOCK_TIME_NONE;
 
+  if (live && min > max) {
+    GST_ELEMENT_WARNING (self, CORE, NEGOTIATION,
+        ("%s", "Latency too big"),
+        ("The requested latency value is too big for the current pipeline. "
+            "Limiting to %" G_GINT64_FORMAT, max));
+    min = max;
+    /* FIXME: This could in theory become negative, but in
+     * that case all is lost anyway */
+    self->priv->latency -= min - max;
+    /* FIXME: shouldn't we g_object_notify() the change here? */
+  }
+
   SRC_BROADCAST (self);
 
   GST_DEBUG_OBJECT (self, "configured latency live:%s min:%" G_GINT64_FORMAT
@@ -1567,22 +1554,18 @@ gst_aggregator_event_forward_func (GstPad * pad, gpointer user_data)
   GstAggregatorPad *aggpad = GST_AGGREGATOR_PAD (pad);
 
   if (peer) {
-    if (evdata->only_to_active_pads && aggpad->priv->first_buffer) {
-      GST_DEBUG_OBJECT (pad, "not sending event to inactive pad");
-      ret = TRUE;
-    } else {
-      ret = gst_pad_send_event (peer, gst_event_ref (evdata->event));
-      GST_DEBUG_OBJECT (pad, "return of event push is %d", ret);
-      gst_object_unref (peer);
-    }
+    ret = gst_pad_send_event (peer, gst_event_ref (evdata->event));
+    GST_DEBUG_OBJECT (pad, "return of event push is %d", ret);
+    gst_object_unref (peer);
   }
 
   if (ret == FALSE) {
+    if (GST_EVENT_TYPE (evdata->event) == GST_EVENT_SEEK)
+      GST_ERROR_OBJECT (pad, "Event %" GST_PTR_FORMAT " failed", evdata->event);
+
     if (GST_EVENT_TYPE (evdata->event) == GST_EVENT_SEEK) {
       GstQuery *seeking = gst_query_new_seeking (GST_FORMAT_TIME);
 
-      GST_DEBUG_OBJECT (pad, "Event %" GST_PTR_FORMAT " failed", evdata->event);
-
       if (gst_pad_query (peer, seeking)) {
         gboolean seekable;
 
@@ -1619,7 +1602,7 @@ gst_aggregator_event_forward_func (GstPad * pad, gpointer user_data)
 
 static EventData
 gst_aggregator_forward_event_to_all_sinkpads (GstAggregator * self,
-    GstEvent * event, gboolean flush, gboolean only_to_active_pads)
+    GstEvent * event, gboolean flush)
 {
   EventData evdata;
 
@@ -1627,7 +1610,6 @@ gst_aggregator_forward_event_to_all_sinkpads (GstAggregator * self,
   evdata.result = TRUE;
   evdata.flush = flush;
   evdata.one_actually_seeked = FALSE;
-  evdata.only_to_active_pads = only_to_active_pads;
 
   /* We first need to set all pads as flushing in a first pass
    * as flush_start flush_stop is sometimes sent synchronously
@@ -1687,8 +1669,7 @@ gst_aggregator_do_seek (GstAggregator * self, GstEvent * event)
   GST_OBJECT_UNLOCK (self);
 
   /* forward the seek upstream */
-  evdata =
-      gst_aggregator_forward_event_to_all_sinkpads (self, event, flush, FALSE);
+  evdata = gst_aggregator_forward_event_to_all_sinkpads (self, event, flush);
   event = NULL;
 
   if (!evdata.result || !evdata.one_actually_seeked) {
@@ -1731,12 +1712,7 @@ gst_aggregator_default_src_event (GstAggregator * self, GstEvent * event)
     }
   }
 
-  /* Don't forward QOS events to pads that had no active buffer yet. Otherwise
-   * they will receive a QOS event that has earliest_time=0 (because we can't
-   * have negative timestamps), and consider their buffer as too late */
-  evdata =
-      gst_aggregator_forward_event_to_all_sinkpads (self, event, FALSE,
-      GST_EVENT_TYPE (event) == GST_EVENT_QOS);
+  evdata = gst_aggregator_forward_event_to_all_sinkpads (self, event, FALSE);
   res = evdata.result;
 
 done:
@@ -1874,7 +1850,7 @@ gst_aggregator_set_latency_property (GstAggregator * self, gint64 latency)
  * Gets the latency value. See gst_aggregator_set_latency for
  * more details.
  *
- * Returns: The time in nanoseconds to wait for data to arrive on a sink pad
+ * Returns: The time in nanoseconds to wait for data to arrive on a sink pad 
  * before a pad is deemed unresponsive. A value of -1 means an
  * unlimited time.
  */
@@ -1957,8 +1933,6 @@ gst_aggregator_class_init (GstAggregatorClass * klass)
   klass->src_event = gst_aggregator_default_src_event;
   klass->src_query = gst_aggregator_default_src_query;
 
-  klass->create_new_pad = gst_aggregator_default_create_new_pad;
-
   gstelement_class->request_new_pad =
       GST_DEBUG_FUNCPTR (gst_aggregator_request_new_pad);
   gstelement_class->send_event = GST_DEBUG_FUNCPTR (gst_aggregator_send_event);
@@ -2013,7 +1987,7 @@ gst_aggregator_init (GstAggregator * self, GstAggregatorClass * klass)
       gst_element_class_get_pad_template (GST_ELEMENT_CLASS (klass), "src");
   g_return_if_fail (pad_template != NULL);
 
-  priv->max_padserial = -1;
+  priv->padcount = -1;
   priv->tags_changed = FALSE;
 
   self->priv->peer_latency_live = FALSE;
@@ -2141,6 +2115,10 @@ gst_aggregator_pad_chain_internal (GstAggregator * self,
   if (aggpad->priv->pending_eos == TRUE)
     goto eos;
 
+  flow_return = aggpad->priv->flow_return;
+  if (flow_return != GST_FLOW_OK)
+    goto flushing;
+
   PAD_UNLOCK (aggpad);
 
   if (aggclass->clip && head) {
@@ -2154,11 +2132,8 @@ gst_aggregator_pad_chain_internal (GstAggregator * self,
 
   buf_pts = GST_BUFFER_PTS (actual_buf);
 
-  aggpad->priv->first_buffer = FALSE;
-
   for (;;) {
     SRC_LOCK (self);
-    GST_OBJECT_LOCK (self);
     PAD_LOCK (aggpad);
     if (gst_aggregator_pad_has_space (self, aggpad)
         && aggpad->priv->flow_return == GST_FLOW_OK) {
@@ -2175,12 +2150,10 @@ gst_aggregator_pad_chain_internal (GstAggregator * self,
 
     flow_return = aggpad->priv->flow_return;
     if (flow_return != GST_FLOW_OK) {
-      GST_OBJECT_UNLOCK (self);
       SRC_UNLOCK (self);
       goto flushing;
     }
     GST_DEBUG_OBJECT (aggpad, "Waiting for buffer to be consumed");
-    GST_OBJECT_UNLOCK (self);
     SRC_UNLOCK (self);
     PAD_WAIT_EVENT (aggpad);
 
@@ -2196,7 +2169,6 @@ gst_aggregator_pad_chain_internal (GstAggregator * self,
         start_time = 0;
         break;
       case GST_AGGREGATOR_START_TIME_SELECTION_FIRST:
-        GST_OBJECT_LOCK (aggpad);
         if (aggpad->segment.format == GST_FORMAT_TIME) {
           start_time = buf_pts;
           if (start_time != -1) {
@@ -2212,7 +2184,6 @@ gst_aggregator_pad_chain_internal (GstAggregator * self,
               "as the segment is a %s segment instead of a time segment",
               gst_format_get_name (aggpad->segment.format));
         }
-        GST_OBJECT_UNLOCK (aggpad);
         break;
       case GST_AGGREGATOR_START_TIME_SELECTION_SET:
         start_time = self->priv->start_time;
@@ -2226,6 +2197,8 @@ gst_aggregator_pad_chain_internal (GstAggregator * self,
         self->segment.position = start_time;
       else
         self->segment.position = MIN (start_time, self->segment.position);
+      self->segment.start = MIN (start_time, self->segment.start);
+      self->segment.time = MIN (start_time, self->segment.time);
 
       GST_DEBUG_OBJECT (self, "Selecting start time %" GST_TIME_FORMAT,
           GST_TIME_ARGS (start_time));
@@ -2233,7 +2206,6 @@ gst_aggregator_pad_chain_internal (GstAggregator * self,
   }
 
   PAD_UNLOCK (aggpad);
-  GST_OBJECT_UNLOCK (self);
   SRC_UNLOCK (self);
 
 done:
@@ -2304,11 +2276,10 @@ flushing:
   return FALSE;
 }
 
-static GstFlowReturn
+static gboolean
 gst_aggregator_pad_event_func (GstPad * pad, GstObject * parent,
     GstEvent * event)
 {
-  GstFlowReturn ret = GST_FLOW_OK;
   GstAggregator *self = GST_AGGREGATOR (parent);
   GstAggregatorPad *aggpad = GST_AGGREGATOR_PAD (pad);
   GstAggregatorClass *klass = GST_AGGREGATOR_GET_CLASS (parent);
@@ -2319,10 +2290,8 @@ gst_aggregator_pad_event_func (GstPad * pad, GstObject * parent,
     PAD_LOCK (aggpad);
 
     if (aggpad->priv->flow_return != GST_FLOW_OK
-        && GST_EVENT_TYPE (event) != GST_EVENT_FLUSH_STOP) {
-      ret = aggpad->priv->flow_return;
+        && GST_EVENT_TYPE (event) != GST_EVENT_FLUSH_STOP)
       goto flushing;
-    }
 
     if (GST_EVENT_TYPE (event) == GST_EVENT_SEGMENT) {
       GST_OBJECT_LOCK (aggpad);
@@ -2344,17 +2313,10 @@ gst_aggregator_pad_event_func (GstPad * pad, GstObject * parent,
     SRC_UNLOCK (self);
   }
 
-  if (event) {
-    gboolean is_caps = (GST_EVENT_TYPE (event) == GST_EVENT_CAPS);
-
-    if (!klass->sink_event (self, aggpad, event)) {
-      /* Copied from GstPad to convert boolean to a GstFlowReturn in
-       * the event handling func */
-      ret = is_caps ? GST_FLOW_NOT_NEGOTIATED : GST_FLOW_ERROR;
-    }
-  }
-
-  return ret;
+  if (event)
+    return klass->sink_event (self, aggpad, event);
+  else
+    return TRUE;
 
 flushing:
   GST_DEBUG_OBJECT (aggpad, "Pad is %s, dropping event",
@@ -2364,8 +2326,7 @@ flushing:
   if (GST_EVENT_IS_STICKY (event))
     gst_pad_store_sticky_event (pad, event);
   gst_event_unref (event);
-
-  return ret;
+  return FALSE;
 }
 
 static gboolean
@@ -2402,9 +2363,8 @@ gst_aggregator_pad_constructed (GObject * object)
 
   gst_pad_set_chain_function (pad,
       GST_DEBUG_FUNCPTR (gst_aggregator_pad_chain));
-  gst_pad_set_event_full_function_full (pad,
-      GST_DEBUG_FUNCPTR (gst_aggregator_pad_event_func),
-      NULL, NULL);
+  gst_pad_set_event_function (pad,
+      GST_DEBUG_FUNCPTR (gst_aggregator_pad_event_func));
   gst_pad_set_query_function (pad,
       GST_DEBUG_FUNCPTR (gst_aggregator_pad_query_func));
   gst_pad_set_activatemode_function (pad,
@@ -2457,8 +2417,6 @@ gst_aggregator_pad_init (GstAggregatorPad * pad)
 
   g_mutex_init (&pad->priv->flush_lock);
   g_mutex_init (&pad->priv->lock);
-
-  pad->priv->first_buffer = TRUE;
 }
 
 /**
