@@ -27,7 +27,6 @@
 #include <gst/gst.h>
 #include <gst/base/gstbasesrc.h>
 #include "gstavdtputil.h"
-#include "gstavrcputil.h"
 
 G_BEGIN_DECLS
 #define GST_TYPE_AVDTP_SRC \
@@ -55,15 +54,9 @@ struct _GstAvdtpSrc
   GstAvdtpConnection conn;
   GstCaps *dev_caps;
 
-  GstAvrcpConnection *avrcp;
-
   GstPoll *poll;
   GstPollFD pfd;
   volatile gint unlocked;
-
-  GstClockTime duration;
-
-  guint transport_volume;
 };
 
 GType gst_avdtp_src_get_type (void);
